// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PixelPerfectCamera.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PixelPerfectCamera::PixelPerfectCamera(SceneItem *i) : SceneItemComponent(i) {

}

std::vector<SceneItemComponent*> PixelPerfectCamera::getDependencies() {
  return std::vector<SceneItemComponent*>{
    (this->camera = this->item->getComponent<Camera>())
  };
}

void PixelPerfectCamera::onRenderTargetResized(float_t w, float_t h) {
  this->updateDimensions();
}

void PixelPerfectCamera::updateDimensions() {
  float_t w, h;
  assertNotNull(this->camera);

  auto target = this->camera->getRenderTarget();
  switch(this->camera->type) {
    case CAMERA_TYPE_ORTHONOGRAPHIC:
      w = target->getWidth() / 2.0f / this->scale;
      h = target->getHeight() / 2.0f / this->scale;
      camera->orthoLeft = -w;
      camera->orthoRight = w;
      camera->orthoTop = h;
      camera->orthoBottom = -h;
      camera->updateProjection();
      break;
    
    case CAMERA_TYPE_PERSPECTIVE:
      this->transform->lookAtPixelPerfect(
        glm::vec3(0, 0, 0),
        glm::vec3(0, 0, 0),
        target->getHeight() / this->scale,
        this->camera->fov
      );
      // this->transform->lookAt(glm::vec3(360, 360, 360), glm::vec3(0, 0, 0));
      break;
    
    default:
      assertUnreachable();
  }
}

void PixelPerfectCamera::onStart() {
  assertNotNull(this->camera);
  this->updateDimensions();
  camera->eventRenderTargetResized.addListener(
    this, &PixelPerfectCamera::onRenderTargetResized
  );
}

void PixelPerfectCamera::onDispose() {
  camera->eventRenderTargetResized.removeListener(
    this, &PixelPerfectCamera::onRenderTargetResized
  );
}